/*
 *  lol_flowctrl.cpp
 *
 *  LOL++, implementation of LOLCODE-like language on top of C++
 *  Copyright (C) 2008-2009 Yusdi Saliman
 *
 *  This file contains the implementation of functions related to flow control.
 *
 *  LOL++ is free software. It comes without any warranty, to
 *  the extent permitted by applicable law. You can redistribute it
 *  and/or modify it under the terms of the Do What The Fuck You Want
 *  To Public License, Version 2, as published by Sam Hocevar. See
 *  http://sam.zoy.org/wtfpl/COPYING for more details.
 */

#include "lol_flowctrl.h"

bool __lol_loopJustStarted;

/** The first time UPPIN and NERFIN is executed in a loop, they do nothing */
void __lol_initLoop()
{
    __lol_loopJustStarted = true;
}

/** increment variable if the type is integer or float */
bool __lol_uppin(__LolData& lol)
{
    if (__lol_loopJustStarted)
    {
        __lol_loopJustStarted = false;
        return true;
    }

    if (lol.getType() == __LOL_NUMBR)
    {
        lol = lol.toInt() + 1;
    }
    else if (lol.getType() == __LOL_NUMBAR)
    {
        lol = lol.toFloat() + 1;
    }
    
    return true;
}

/** decrement variable if the type is integer or float */
bool __lol_nerfin(__LolData& lol)
{
    if (__lol_loopJustStarted)
    {
        __lol_loopJustStarted = false;
        return true;
    }

    if (lol.getType() == __LOL_NUMBR)
        lol = lol.toInt() - 1;
    else if (lol.getType() == __LOL_NUMBAR)
        lol = lol.toFloat() - 1;
    
    return true;
}
