/*
 *  lol_io.cpp
 *
 *  LOL++, implementation of LOLCODE-like language on top of C++
 *  Copyright (C) 2008-2009 Yusdi Saliman
 *
 *  This file contains the implementation of I/O functions.
 *
 *  LOL++ is free software. It comes without any warranty, to
 *  the extent permitted by applicable law. You can redistribute it
 *  and/or modify it under the terms of the Do What The Fuck You Want
 *  To Public License, Version 2, as published by Sam Hocevar. See
 *  http://sam.zoy.org/wtfpl/COPYING for more details.
 */

#include "lol_io.h"
#include <iostream>
using namespace std;

/** Print the string representation of a data. The data itself doesn't have
  * to be a YARN. */
void __lol_visible(__LolData lol, bool addNewLine)
{
    string str = lol.toString();
    int i = 0;
    bool escapeCharFound = false;
    char c;
    while (c = str[i++])
    {
        if (escapeCharFound)
        {
            switch (c)
            {
                case __LOL_NEWLINE:
                    cout << endl;
                    break;
                case __LOL_TAB:
                    cout << "\t";
                    break;
                case __LOL_COLON:
                    cout << ":";
                    break;
                case __LOL_BELL:
                    cout << "\a";
                    break;
                default:
                    cout << c;
            }
            escapeCharFound = false;
        }
        else
        {
            if (c == __LOL_ESC_CHAR)
                escapeCharFound = true;
            else
                cout << c;
        }
    }
    
    if (addNewLine)
        cout << endl;
}

/** __LolData is actually capable of handling different input types, but since
  * the current LOLCODE spec (1.2) says that GIMMEH only accepts YARN type,
  * the resulting variable will always be cast to YARN. */
void __lol_gimmeh(__LolData& lol)
{
    string str;
    cin >> str;
    lol = str;
}
