/*
 *  lol_operator.cpp
 *
 *  LOL++, implementation of LOLCODE-like language on top of C++
 *  Copyright (C) 2008-2009 Yusdi Saliman
 *
 *  This file contains the implementation of LOL++ operator functions.
 *
 *  LOL++ is free software. It comes without any warranty, to
 *  the extent permitted by applicable law. You can redistribute it
 *  and/or modify it under the terms of the Do What The Fuck You Want
 *  To Public License, Version 2, as published by Sam Hocevar. See
 *  http://sam.zoy.org/wtfpl/COPYING for more details.
 */

#include "lol_operator.h"

/** return data1 + data2 if both are float or integer */
__LolData __lol_sum(__LolData data1, __LolData data2)
{
    data1.add(data2);
    __lol_setIt(data1);
    return __lol_it();
}

/** return data1 - data2 if both are float or integer */
__LolData __lol_diff(__LolData data1, __LolData data2)
{
    data1.sub(data2);
    __lol_setIt(data1);
    return __lol_it();
}

/** return data1 * data2 if both are float or integer */
__LolData __lol_produkt(__LolData data1, __LolData data2)
{
    data1.multiply(data2);
    __lol_setIt(data1);
    return __lol_it();
}

/** return data1 / data2 if both are float or integer */
__LolData __lol_quoshunt(__LolData data1, __LolData data2)
{
    data1.divide(data2);
    __lol_setIt(data1);
    return __lol_it();
}

/** return data1 mod data2 if both are integers */
__LolData __lol_mod(__LolData data1, __LolData data2)
{
    data1.mod(data2);
    __lol_setIt(data1);
    return __lol_it();
}

/** return data1 > data2 ? data1 : data2 */
__LolData __lol_biggr(__LolData data1, __LolData data2)
{
    // cast everything to floating point for comparison
    __lol_setIt(data1.toFloat() > data2.toFloat() ? data1 : data2);
    return __lol_it();
}

/** return data1 < data2 ? data1 : data2 */
__LolData __lol_smallr(__LolData data1, __LolData data2)
{
    // cast everything to floating point for comparison
    __lol_setIt(data1.toFloat() < data2.toFloat() ? data1 : data2);
    return __lol_it();
}

/** return data1 AND data2.
  * If changeIt is false, IT will not be modified. This is a hack to make
  * BOTH SAEM works. */
__LolData __lol_both(__LolData data1, __LolData data2, bool changeIt)
{
    bool b = data1.toBool() && data2.toBool();
    if (changeIt)
    {
        __lol_setIt(b);
        return __lol_it();
    }
    else
    {
        __LolData lol = b;
        return lol;
    }
}

/** return data1 OR data2 */
__LolData __lol_either(__LolData data1, __LolData data2)
{
    __lol_setIt(data1.toBool() || data2.toBool());
    return __lol_it();
}

/** return data1 XOR data2 */
__LolData __lol_won(__LolData data1, __LolData data2)
{
    __lol_setIt((bool) (data1.toBool() ^ data2.toBool()));
    return __lol_it();
}

/** return NOT data */
__LolData __lol_not(__LolData data)
{
    __lol_setIt(!data.toBool());
    return __lol_it();
}

/** return true if all data are true. Max number of data is 8,
  * parameter 3-8 are optional. */
__LolData __lol_all(__LolData data1, __LolData data2,
                    __LolData data3, __LolData data4,
                    __LolData data5, __LolData data6,
                    __LolData data7, __LolData data8)
{
    __lol_setIt(__lol_both(data1, data2));
    __lol_setIt(__lol_both(__lol_it(), data3));
    __lol_setIt(__lol_both(__lol_it(), data4));
    __lol_setIt(__lol_both(__lol_it(), data5));
    __lol_setIt(__lol_both(__lol_it(), data6));
    __lol_setIt(__lol_both(__lol_it(), data7));
    __lol_setIt(__lol_both(__lol_it(), data8));
    return __lol_it();
}

/** return true if any of the data is true. Max number of data is 8,
  * parameter 3-8 are optional. */
__LolData __lol_any(__LolData data1, __LolData data2,
                    __LolData data3, __LolData data4,
                    __LolData data5, __LolData data6,
                    __LolData data7, __LolData data8)
{
    __lol_setIt(__lol_either(data1, data2));
    __lol_setIt(__lol_either(__lol_it(), data3));
    __lol_setIt(__lol_either(__lol_it(), data4));
    __lol_setIt(__lol_either(__lol_it(), data5));
    __lol_setIt(__lol_either(__lol_it(), data6));
    __lol_setIt(__lol_either(__lol_it(), data7));
    __lol_setIt(__lol_either(__lol_it(), data8));
    return __lol_it();
}

/** return true if data1 == data2 */
__LolData __lol_both_saem(__LolData data1, __LolData data2)
{
    __lol_setIt(data1 == data2);
    return __lol_it();
}

/** return true if data1 != data2 */
__LolData __lol_diffrint(__LolData data1, __LolData data2)
{
    __lol_setIt(data1 != data2);
    return __lol_it();
}

/** concatenate strings. Max number of strings is 8,
  * parameter 3-8 are optional. */
__LolData __lol_smoosh(__LolData data1, __LolData data2,
                       __LolData data3, __LolData data4,
                       __LolData data5, __LolData data6,
                       __LolData data7, __LolData data8)
{
    string str = data1.toString();
    str.append(data2.toString());
    str.append(data3.toString());
    str.append(data4.toString());
    str.append(data5.toString());
    str.append(data6.toString());
    str.append(data7.toString());
    str.append(data8.toString());
    __lol_setIt(str);
    return __lol_it();
}

/** casting */
__LolData __lol_maek(__LolData data, __LOL_TYPES type)
{
    data.castTo(type);
    return data;
}
