/*
 *  lol_types.h
 *  
 *  LOL++, implementation of LOLCODE-like language on top of C++
 *  Copyright (C) 2008-2009 Yusdi Saliman
 *
 *  This file contains the declaration of the C++ class that emulates
 *  LOLCODE's data types.
 *
 *  LOL++ is free software. It comes without any warranty, to
 *  the extent permitted by applicable law. You can redistribute it
 *  and/or modify it under the terms of the Do What The Fuck You Want
 *  To Public License, Version 2, as published by Sam Hocevar. See
 *  http://sam.zoy.org/wtfpl/COPYING for more details.
 */

#ifndef __LOL_TYPES_H
#define __LOL_TYPES_H

#ifdef __LOL_TYPES_CPP
#define __X_LOL_TYPES
#else
#define __X_LOL_TYPES   extern
#endif

#include <string>
using namespace std;

enum __LOL_TYPES
    { __LOL_NOOB, __LOL_YARN, __LOL_NUMBR, __LOL_NUMBAR, __LOL_TROOF };

class __LolData
{
public:
	// constructors
    __LolData():type(__LOL_NOOB) { pArray = NULL; size = 0; }
    __LolData(const char* str);
    __LolData(string str);
    __LolData(int n);
    __LolData(double f);
    __LolData(bool b);
    
    // since the array is dynamically allocated, we need a copy constructor
    // and overloaded assignment operator
    __LolData(const __LolData& lol);
    __LolData& operator= (const __LolData lol);
    __LolData& operator= (const char* str);
    __LolData& operator= (string str);
    __LolData& operator= (int n);
    __LolData& operator= (double f);
    __LolData& operator= (bool b);

    // destructor
    ~__LolData()    { if (pArray) delete [] pArray; }
    
    // overload comparison operators
    bool operator== (const __LolData& lol);
    bool operator== (const char* str);
    bool operator== (int n);
    bool operator== (double f);
    bool operator== (bool b);
    bool operator!= (const __LolData& lol);
    bool operator!= (const char* str);
    bool operator!= (int n);
    bool operator!= (double f);
    bool operator!= (bool b);
    
    // add operator&& to make BOTH SAEM works
    __LolData operator&&(__LolData lol);

    // add operator& as a string concatenator
    __LolData operator&(__LolData lol);
    
    // mathematical operations
    void add(__LolData lol);
    void sub(__LolData lol);
    void multiply(__LolData lol);
    void divide(__LolData lol);
    void mod(__LolData lol);
    
    // methods for casting and reading value
    void castTo(__LOL_TYPES t);
    string toString();
    int toInt();
    double toFloat();
    bool toBool();
    
    // accessor method
    __LOL_TYPES getType() const { return type; }

    ////// for array
    
    // operator[]
    __LolData& operator[] (__LolData index);

    // methods
    __LolData getLength() const { __LolData lol(size); return lol; }

private:
    // attributes
    __LOL_TYPES type;
    string strVal;
    int intVal;
    double floatVal;
    bool boolVal;

    // array
    __LolData* pArray;
    int size;
    
    // private method
    bool checkTypes(__LolData& lol);
    void createArray(__LolData s);
    void resizeArray(__LolData s);
};

// the LOLCODE special variable IT
__X_LOL_TYPES __LolData __lol_it_var;

// to access the variable, use the macros below. This will make sure that we can
// change them into functions should the need arise one day. 
#define __lol_it()  __lol_it_var
#define __lol_setIt(x)  __lol_it_var = x

#endif
