/*
 *  lol_operator_test.cpp
 *  
 *  LOL++, implementation of LOLCODE-like language on top of C++
 *  Copyright (C) 2008-2009 Yusdi Saliman
 *
 *  This file contains a simple unit test program to test operator functions
 *
 *  LOL++ is free software. It comes without any warranty, to
 *  the extent permitted by applicable law. You can redistribute it
 *  and/or modify it under the terms of the Do What The Fuck You Want
 *  To Public License, Version 2, as published by Sam Hocevar. See
 *  http://sam.zoy.org/wtfpl/COPYING for more details.
 */

#include "lol_operator.h"

#include <iostream>
#include <cassert>
using namespace std;

int main()
{
    // integer tests
    __LolData data1 = 10;
    __LolData data2 = 20;
    
    assert(__lol_sum(data1, data2) == 30);
    assert(__lol_diff(data2, data1) == 10);
    assert(__lol_produkt(data1, data2) == 200);
    assert(__lol_quoshunt(data2, data1) == 2);
    assert(__lol_mod(data2, data1) == 0);
    assert(__lol_biggr(data1, data2) == 20);
    assert(__lol_smallr(data1, data2) == 10);
    
    // float tests
    data1 = 0.5;
    data2 = 0.25;
    
    assert(__lol_sum(data1, data2) == 0.75);
    assert(__lol_diff(data1, data2) == 0.25);
    assert(__lol_produkt(data1, data2) == 0.125);
    assert(__lol_quoshunt(data1, data2) == 2);
    assert(__lol_biggr(data1, data2) == 0.5);
    assert(__lol_smallr(data1, data2) == 0.25);
    
    // float and integer tests
    data1 = 10;
    data2 = 0.5;
    
    assert(__lol_sum(data1, data2) == 10.5);
    assert(__lol_diff(data1, data2) == 9.5);
    assert(__lol_produkt(data1, data2) == 5);
    assert(__lol_quoshunt(data1, data2) == 20);
    assert(__lol_biggr(data1, data2) == 10);
    assert(__lol_smallr(data1, data2) == 0.5);
    
    // boolean tests
    assert(__lol_both_saem(data1, data2) == false);
    assert(__lol_diffrint(data1, data2) == true);
    data2 = 10;
    assert(__lol_both_saem(data1, data2) == true);
    assert(__lol_diffrint(data1, data2) == false);
    assert((__lol_both(true,true,false) && __lol_both_saem(data1,data2))==true);
    
    data1 = true;
    data2 = false;
    __LolData data3 = true;
    __LolData data4 = false;
    
    assert(__lol_both(data1, data2) == false);
    assert(__lol_either(data1, data2) == true);
    assert(__lol_won(data1, data2) == true);
    assert(__lol_not(data1) == false);
    assert(__lol_all(data1, data2, data3, data4) == false);
    assert(__lol_any(data1, data2, data3, data4) == true);
    
    // string concatenation test
    data1 = "test1 ";
    data2 = "test2";
    assert(__lol_smoosh(data1, data2) == "test1 test2");
    
    // casting test
    data1 = 10;
    assert(__lol_maek(data1, __LOL_NUMBAR) == 10.0);
    assert(__lol_maek(data1, __LOL_YARN) == "10");
    assert(__lol_maek(data1, __LOL_TROOF).getType() == __LOL_TROOF);
    assert(__lol_maek(data1, __LOL_NOOB).getType() == __LOL_NOOB);
    
    cout << "All tests passed.\n";
    return 0;
}
