/*
 *  lol_types_test.cpp
 *  
 *  LOL++, implementation of LOLCODE-like language on top of C++
 *  Copyright (C) 2008-2009 Yusdi Saliman
 *
 *  This file contains a simple unit test program to test data types
 *
 *  LOL++ is free software. It comes without any warranty, to
 *  the extent permitted by applicable law. You can redistribute it
 *  and/or modify it under the terms of the Do What The Fuck You Want
 *  To Public License, Version 2, as published by Sam Hocevar. See
 *  http://sam.zoy.org/wtfpl/COPYING for more details.
 */

#include "lol_types.h"

#include <iostream>
#include <cassert>
using namespace std;

int main()
{
    __LolData data1;
    
    // initialized type is NOOB
    assert(data1.getType() == __LOL_NOOB);
    
    // test integer assignment
    data1 = 10;
    assert(data1.getType() == __LOL_NUMBR);
    assert(data1.toInt() == 10);
    
    // test string assignment
    data1 = "test";
    assert(data1.getType() == __LOL_YARN);
    assert(data1.toString() == "test");
    
    // test floating point number assignment
    data1 = 1.2;
    assert(data1.getType() == __LOL_NUMBAR);
    assert(data1.toFloat() == 1.2);
    
    // test boolean assignment
    data1 = true;
    assert(data1.getType() == __LOL_TROOF);
    assert(data1.toBool() == true);
    
    // test object assignment
    __LolData data2 = data1;
    assert(data2.getType() == data1.getType());
    assert(data2.toString() == data1.toString());
    assert(data2.toInt() == data1.toInt());
    assert(data2.toFloat() == data1.toFloat());
    assert(data2.toBool() == data1.toBool());
    
    // test object comparison
    assert(data1 == data2);
    data1 = false;
    assert(data1 != data2);
    data1 = 100;
    assert(data1 != data2);
    
    // test string comparison
    data1 = "test";
    assert(data1 == "test");
    assert(data1 != "abcde");
    
    // test integer comparison
    data1 = 2;
    assert(data1 == 2);
    assert(data1 != 20);
    
    // test floating point comparison
    data1 = 20.2;
    assert(data1 == 20.2);
    assert(data1 != 20);
    
    // test boolean comparison
    data1 = false;
    assert(data1 == false);
    assert(data1 != true);
    
    // test mathematical functions
    data1 = 10;
    data1.add(20);
    assert(data1 == 30);
    data1.sub(1.5);
    assert(data1 == 28.5);
    data1.multiply(10);
    assert(data1 == 285);
    data1 = 10;
    data1.divide(2);
    assert(data1 == 5);
    data1.mod(2);
    assert(data1 == 1);

    // test string concatenation
    data1 = "string ";
    data2 = "concatenation";
    assert((data1 & data2) == "string concatenation");
    assert((data1 & "concatenation") == "string concatenation");
    assert((data1 & 1) == "string 1");

    // test array
    __LolData array;
    assert(array.getLength() == 0);
    array[1] = 10;
    assert(array.getLength() == 1);
    assert(array[1] == 10);
    array[3] = 20;
    assert(array.getLength() == 3);
    assert(array[2].getType() == __LOL_NOOB);
    assert(array[3] == 20);
    
    cout << "All tests passed.\n";
    return 0;
}
